package CVSMonitor::Backend::DataCache;

# This package implements a comprehensive data cache, holding all
# information retrieved from the repository that is not the main
# history.

use strict;
use UNIVERSAL 'isa';
use File::Spec ();
use Class::Autouse qw{File::Flat};





#####################################################################
# Interface Methods

sub exists {
	my $class = shift;
	my $Version = shift or return undef;
	my $resource = shift or return undef;
	my $filename = $class->_filename( $Version, $resource ) or return undef;

	-e $filename ? 1 : 0;
}

sub get {
	my $class = shift;
	my $Version = shift or return undef;
	my $resource = shift or return undef;
	my $filename = $class->_filename( $Version, $resource ) or return undef;

	# Make sure the file exists
	return undef unless -e $filename;

	# Slurp in the file
	File::Flat->slurp( $filename );
}

sub set {
	my $class = shift;
	my $Version = shift or return undef;
	my $resource = shift or return undef;
	my $content = shift or return undef;
	my $filename = $class->_filename( $Version, $resource ) or return undef;

	# Can we write the file
	return undef unless File::Flat->canWrite( $filename );

	# Write the file
	File::Flat->write( $filename, $content ) ? 1 : undef;
}




 
#####################################################################
# Non-Interface Methods

sub _filename {
	my $class = shift;
	my $Version = isa( $_[0], 'CVSMonitor::MetaData::Version' )
		? shift : return undef;
	my $resource = shift or return undef;

	# Build the full file path
	my $filename = $Version->File->filename 
		. '-' . $resource
		. '-' . $Version->revision;
	File::Spec->catfile( $Version->Module->pathCvsCacheDir, $filename );
}

1;

__END__

# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
