package CVSMonitor::Graph::Files;

# This package implements a Graph that calculate statistics on the quantity
# of files over time.

use strict;
use UNIVERSAL 'isa';
use base 'CVSMonitor::Graph::Range';
use Class::Autouse 'GD::Graph::lines',
                   'Chart::Math::Axis';
                   
                   
                   
                   

# What sort of Range do we need?
sub rangeClass { 'CVSMonitor::Range::Files' }

# Returns the GD::Image object for the graph
sub gd {
	my $self = shift;

	# Build the datasets
	my $dates = $self->{Range}->slice( 'date' ) or return undef;
	my $up = $self->{Range}->slice( 'added' ) or return undef;
	my $down = $self->{Range}->slice( 'removed' ) or return undef;
	$down = $self->negateSlice( $down ) or return undef;
	my $size = $self->addSlices( $up, $down ) or return undef;
	my $accum = $self->accumulateSlice( $size );

	# Add the start data if needed
	if ( $self->getShowStart ) {
		unshift @$dates, 'Start';
		unshift @$up, 0; unshift @$down, 0;
		unshift @$size, 0; unshift @$accum, 0;
	}

	# Create the graph object
	my $graph = GD::Graph::lines->new( $self->getWidth, $self->getHeight ) or return undef;

	# Set the legend
	$graph->set_legend( 'Added', 'Removed', 'Difference', 'Total Files' );

	# Since the axis provided by GD::Graph sucks, we will generate our own
	# Use the oh-so-nifty Chart::Math::Axis to acomplish this.
	my $y_max_value = $self->sliceMax( $up, $down, $size, $accum );
	my $y_min_value = $self->sliceMin( $up, $down, $size, $accum );
	my $Axis = Chart::Math::Axis->new( $y_max_value, $y_min_value ) or return undef;
	$Axis->include_zero() or return undef;
	if ( $self->{maximum_intervals} ) {
		$Axis->set_maximum_intervals( $self->{maximum_intervals} );
	}
	$Axis->apply_to( $graph );

	# Set the calculated graph properties
	$graph->set( 
		title             => $self->getTitle,
		x_labels_vertical => 1,
		y_label           => 'Files',
		dclrs             => [ qw{green red blue black} ],
		legend_placement  => 'BC',
		zero_axis         => 1,
		skip_undef        => 1,
		);

	# Plot and return
	$graph->plot( [ $dates, $up, $down, $size, $accum ] );
}

1;

__END__

# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
