package CVSMonitor::HTML;

use strict;
use UNIVERSAL 'isa';
use Clone ();





#####################################################################
# Escaping Methods

sub escape {
	my $html = defined $_[1] ? "$_[1]" : return '';
	$html =~ s/&/&amp;/sgo;
	$html =~ s/</&lt;/sgo;
	$html =~ s/>/&gt;/sgo;
	$html =~ s/\"/&quot;/sgo;
	$html;
}

sub escape_multiline {
	my $html = defined $_[1] ? "$_[1]" : return '';
	$html =~ s/&/&amp;/sgo;
	$html =~ s/</&lt;/sgo;
	$html =~ s/>/&gt;/sgo;
	$html =~ s/\"/&quot;/sgo;
	$html =~ s/(\015\012|\015|\012)/$1<br>/g;
	$html;
}

sub escape_textarea {
	my $html = defined $_[1] ? "$_[1]" : return '';
	$html =~ s/&/&amp;/sgo;
	$html =~ s/</&lt;/sgo;
	$html =~ s/>/&gt;/sgo;
	$html =~ s/\"/&quot;/sgo;
	$html =~ s/\'/&#039;/sgo;
	$html;
}





#####################################################################
# HTML Generators

# Create a generic callback link.
# Don't escape, that is left to the caller
sub callback {
	my $class   = shift;
	my $text    = shift;
	my $args    = ref $_[0] eq 'HASH' ? Clone::clone(shift) : {};
	my $options = ref $_[0] eq 'HASH' ? Clone::clone(shift) : {};

	# Create the href
	my $query_string = join ';',
		map { defined $args->{$_} ? "$_=$args->{$_}" : "$_=" } # Avoid a warning
		keys %$args;
	my $href = CGI::script_name() . CGI::path_info();
	$href .= "?$query_string" if $query_string;

	# Clean the options and add the href
	my $cleaned = $class->_link_options( $options ) or return undef;
	$cleaned->{href} = $href;

	$class->tag_pair( 'a', $cleaned, $text );
}

# As above, but for a generic link
sub link {
	my $class   = shift;
	my $href    = shift or return undef;
	my $text    = CVSMonitor::Util::string($_[0]) ? shift : return undef;
	my $args    = ref $_[0] eq 'HASH' ? shift : {};
	my $options = ref $_[0] eq 'HASH' ? shift : {};

	# Add the arguments to the href
	if ( %$args ) {
		$href .= '?' . join ';',
			map { defined $args->{$_} ? "$_=$args->{$_}" : "$_=" } # Avoid a warning
			keys %$args;
	}

	# Clean the options and add the href
	my $cleaned = $class->_link_options( $options ) or return undef;
	$cleaned->{href} = $href;

	$class->tag_pair( 'a', $cleaned, $text );
}

# A "relink" is a link that starts with the exact params from the current
# call, and then "diffs" it, based on additional criteria
sub relink {
	my $class   = shift;
	my $text    = CVSMonitor::Util::string($_[0]) ? shift : return undef;
	my $diff    = ref $_[0] eq 'HASH' ? Clone::clone(shift) : {};
	my $options = ref $_[0] eq 'HASH' ? Clone::clone(shift) : {};

	# Get the current query and diff it
	my $query = AppLib::CGI->new;
	foreach my $name ( %$diff ) {
		if ( ref $diff->{$name} eq 'ARRAY' ) {
			# Set the param to a list of values
			$query->param( $name, @{$diff->{$name}} );
		} elsif ( defined $diff->{$name} ) {
			# Set the param to a single value
			$query->param( $name, $diff->{$name} );
		} else {
			# undef means "delete the param"
			$query->delete( $name );
		}
	}

	# Clean the options and add the href
	my $cleaned = $class->_link_options( $options ) or return undef;
	$cleaned->{href} = $query->self_url;

	$class->tag_pair( 'a', $cleaned, $text );
}

# Clean link options
sub _link_options {
	my $class = shift;
	my $options = ref $_[0] eq 'HASH' ? shift : return undef;

	# Lowercase and escape any options
	my %cleaned = ();
	foreach my $option ( keys %$options ) {
		$cleaned{lc $option} = $class->escape($options->{$option});
	}

	# Handle the confirm message option
	if ( $cleaned{confirm} ) {
		$cleaned{onclick} = 'return confirm(&quot;' . $cleaned{confirm} . '&quot;);';
		delete $cleaned{confirm};
	}

	\%cleaned;
}

# Create a label for something
sub label {
	my $class = shift;
	my $props = @_ > 1 ? { for => shift } : {};
	my $content = shift or return undef;
	CVSMonitor::HTML->tag_pair( 'label', $props, $content );
}

sub tag {
	my $class = shift;
	my $tag = shift or return undef;
	my %args = ref $_[0] eq 'HASH' ? %{shift()} : return undef;
	$tag = join ' ', $tag,
		map { defined $args{$_} ? "$_=\"$args{$_}\"" : "$_" } # Avoid a warning
		sort keys %args;
	"<$tag />";
}

sub tag_pair {
	my $class = shift;
	my $tag = shift or return undef;
	my %args = ref $_[0] eq 'HASH' ? %{shift()} : return undef;
	my $content = join '', @_;
	my $opentag = join ' ', $tag,
		map { defined $args{$_} ? "$_=\"$args{$_}\"" : "$_" } # Avoid a warning
		sort keys %args;
	"<$opentag>$content</$tag>";
}

1;

__END__

# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
