package setup::Tests;

# This package contains the standard test battery we use.

use strict;
use UNIVERSAL 'isa';
use ExtUtils::MakeMaker;

use vars qw{@ISA};
BEGIN { @ISA = 'setup::Error' }





#####################################################################
# Tests, loosely grouped by section

# Is the version of perl new enough
sub perlVersion {
	( $] > $_[1] ) ? 1 : 0;
}

# Is a module installed
sub perlModule {
	my $class = shift;
	my $module = shift or return 1;

	# Can we find a file for this module
	my $file = Class::Inspector->resolved_filename( $module ) or return 0;

	# Return the version of the file, if we can tell
	MM->parse_version( $file ) || -1;
}

# Tests if something will actually be called when we
# try to run a certain program.
sub programInstalled {
	my $class = shift;
	my $program = shift or return 1;
	defined File::Which::which( $program ) ? 1 : 0;
}

# Does a file exist
sub filesystemExists {
	my $class = shift;
	my $location = shift;
	-e $location ? 1 : 0
}

# ... and can we read it
sub filesystemCanRead {
	my $class = shift;
	my $location = -e $_[0] ? shift : return 0;
	-d $location
		? (-x $location and -r $location) ? 1 : 0
		: (-r $location) ? 1 : 0;
}

# ... or write it
sub filesystemCanWrite { 
	my $class = shift;
	my $location = -e $_[0] ? shift : return 0;
	-d $location
		? (-x $location and -w $location) ? 1 : 0
		: (-w $location) ? 1 : 0;
}

# ... or both
sub filesystemCanReadWrite { 
	my $class = shift;
	my $location = -e $_[0] ? shift : return 0;
	-d $location
		? (-x $location and -r $location and -w $location) ? 1 : 0
		: (-r $location and -w $location) ? 1 : 0;
}

# ... or can we execute it
sub filesystemCanExecute {
	my $class = shift;
	my $location = -e $_[0] ? shift : return 0;
	-d $location
		? (-x $location) ? 1 : 0
		: (-r $location and -x $location) ? 1 : 0;
}

1;
